# import libraries
from flask import Flask, render_template, Response, flash, redirect, url_for, session, request, logging
import db
from statsmodels.tsa.arima_model import ARIMAResults
import datetime as DT
import pandas as pd
import werkzeug
from collections import Counter
from WordDetection.video_to_audio import convert_video_to_audio
from WordDetection.audio_to_text import convert_audio_to_text
import random
import json
import time

import torch

from ChatBot.model import NeuralNet
from ChatBot.nltk_utils import bag_of_words, tokenize

device = torch.device('cuda' if torch.cuda.is_available() else 'cpu')
import cv2
import os
import numpy as np

from FaceRecognition.real_time_face_recognition import detect_face

today = DT.date.today()

start_date = today + DT.timedelta(days=1)
end_date = today + DT.timedelta(days=7)

db = db.DB()
current_user = ''

with open('ChatBot/intents.json', 'r') as json_data:
    intents = json.load(json_data)

FILE = "ChatBot/last_model.pth"
data = torch.load(FILE)

input_size = data["input_size"]
hidden_size = data["hidden_size"]
output_size = data["output_size"]
all_words = data['all_words']
tags = data['tags']
model_state = data["model_state"]

chat_history = []

model = NeuralNet(input_size, hidden_size, output_size).to(device)
model.load_state_dict(model_state)
model.eval()

bot_name = "Bot"

# Initialize the flask App
app = Flask(__name__)
app.config['PERMANENT_SESSION_LIFETIME'] = DT.timedelta(minutes=5)


# page links
@app.route('/')
def login():
    return render_template('login.html')


@app.route('/register')
def register():
    return render_template('register.html')


@app.route('/face_recognition', methods=['GET'])
def face_recognition():
    return render_template('face_recognition.html')


@app.route('/home')
def home():
    return render_template('home.html')


@app.route('/content_creator')
def content_creator():
    df = pd.read_csv('Prediction/data/topics.csv', index_col='Date', parse_dates=True)
    df = df.dropna()

    topic_1 = ARIMAResults.load('Prediction/models/covid.pkl')
    topic_2 = ARIMAResults.load('Prediction/models/gas.pkl')
    topic_3 = ARIMAResults.load('Prediction/models/politics.pkl')
    topic_4 = ARIMAResults.load('Prediction/models/yohani.pkl')
    topic_5 = ARIMAResults.load('Prediction/models/travelling.pkl')
    topic_6 = ARIMAResults.load('Prediction/models/youtube.pkl')
    index_future_dates = pd.date_range(start=str(start_date), end=str(end_date))

    pred_1 = topic_1.predict(start=len(df), end=len(df) + 6, typ='levels').rename('ARIMA Predictions')
    pred_2 = topic_2.predict(start=len(df), end=len(df) + 6, typ='levels').rename('ARIMA Predictions')
    pred_3 = topic_3.predict(start=len(df), end=len(df) + 6, typ='levels').rename('ARIMA Predictions')
    pred_4 = topic_4.predict(start=len(df), end=len(df) + 6, typ='levels').rename('ARIMA Predictions')
    pred_5 = topic_5.predict(start=len(df), end=len(df) + 6, typ='levels').rename('ARIMA Predictions')
    pred_6 = topic_6.predict(start=len(df), end=len(df) + 6, typ='levels').rename('ARIMA Predictions')

    pred_1.index = index_future_dates
    pred_2.index = index_future_dates
    pred_3.index = index_future_dates
    pred_4.index = index_future_dates
    pred_5.index = index_future_dates
    pred_6.index = index_future_dates

    return_value = []
    value1 = 0
    value2 = 0
    value3 = 0
    value4 = 0
    value5 = 0
    value6 = 0
    for i in range(0, 7):
        value1 += pred_1[i]
        value2 += pred_2[i]
        value3 += pred_3[i]
        value4 += pred_3[i]
        value5 += pred_3[i]
        value6 += pred_3[i]

    return_value.append(round(value1 / 7, 2))
    return_value.append(round(value2 / 7, 2))
    return_value.append(round(value3 / 7, 2))
    return_value.append(round(value4 / 7, 2))
    return_value.append(round(value5 / 7, 2))
    return_value.append(round(value6 / 7, 2))

    return render_template('content_creator.html', data=return_value)


@app.route('/video_analyzer', methods=['GET', 'POST'])
def video_analyzer():
    if request.method == 'POST':
        video = request.files['video']
        filename = werkzeug.utils.secure_filename(video.filename)
        if video.filename != '':
            video.save('WordDetection/videos/' + filename)

            convert_video_to_audio(filename)
            print('video to audio process completed')
            convert_audio_to_text()
            print('audio to text process completed')

            removable_text = ['i', 'me', 'my', 'myself', 'we', 'our', 'ours', 'ourselves', 'you', "you're", "you've", "you'll", "you'd", 'your', 'yours', 'yourself', 'yourselves', 'he', 'him', 'his', 'himself', 'she', "she's", 'her', 'hers', 'herself', 'it', "it's", 'its', 'itself', 'they', 'them', 'their', 'theirs', 'themselves', 'what', 'which', 'who', 'whom', 'this', 'that', "that'll", 'these', 'those', 'am', 'is', 'are', 'was', 'were', 'be', 'been', 'being', 'have', 'has', 'had', 'having', 'do', 'does', 'did', 'doing', 'a', 'an', 'the', 'and', 'but', 'if', 'or', 'because', 'as', 'until', 'while', 'of', 'at', 'by', 'for', 'with', 'about', 'against', 'between', 'into', 'through', 'during', 'before', 'after', 'above', 'below', 'to', 'from', 'up', 'down', 'in', 'out', 'on', 'off', 'over', 'under', 'again', 'further', 'then', 'once', 'here', 'there', 'when', 'where', 'why', 'how', 'all', 'any', 'both', 'each', 'few', 'more', 'most', 'other', 'some', 'such', 'no', 'nor', 'not', 'only', 'own', 'same', 'so', 'than', 'too', 'very', 's', 't', 'can', 'will', 'just', 'don', "don't", 'should', "should've", 'now', 'd', 'll', 'm', 'o', 're', 've', 'y', 'ain', 'aren', "aren't", 'couldn', "couldn't", 'didn', "didn't", 'doesn', "doesn't", 'hadn', "hadn't", 'hasn', "hasn't", 'haven', "haven't", 'isn', "isn't", 'ma', 'mightn', "mightn't", 'mustn', "mustn't", 'needn', "needn't", 'shan', "shan't", 'shouldn', "shouldn't", 'wasn', "wasn't", 'weren', "weren't", 'won', "won't", 'wouldn', "wouldn't"]

            f = open("output.txt", "r")
            # print(f.read())
            data_set = f.read()
            split_it = data_set.split()

            for text in removable_text:
                while text in split_it:
                    split_it.remove(text)

            # Pass the split_it list to instance of Counter class.
            counter = Counter(split_it)
            print(counter)
            # most_common() produces k frequently encountered
            # input values and their respective counts.
            most_occur = counter.most_common()

            anaz_list = []

            if len(most_occur) < 10:
                for i in range(len(most_occur)):
                    anaz_list.append(most_occur[i][0])
            else:
                for i in range(10):
                    anaz_list.append(most_occur[i][0])

            print(most_occur)

            return render_template('video_analyzer.html', status='File Uploaded.. Please wait', all_content=split_it,
                                   analyzed_content=anaz_list, sub_topic_one='All Content',
                                   sub_topic_two='Analyed Content')
        else:
            return render_template('video_analyzer.html', status='File Not allowed')
    else:
        return render_template('video_analyzer.html')


@app.route('/chat_bot', methods=['GET', 'POST'])
def chat_bot():
    if request.method == 'POST':
        msg = request.form['msg']
        chat_history.append('You : ' + msg)
        sentence = tokenize(msg)
        X = bag_of_words(sentence, all_words)
        X = X.reshape(1, X.shape[0])
        X = torch.from_numpy(X).to(device)

        output = model(X)
        _, predicted = torch.max(output, dim=1)

        tag = tags[predicted.item()]

        probs = torch.softmax(output, dim=1)
        prob = probs[0][predicted.item()]
        if prob.item() > 0.75:
            for intent in intents['intents']:
                if tag == intent["tag"]:
                    rand = random.choice(intent['responses'])
                    # chat_history.append('Bot : ' + random.choice(intent['responses']))
                    if rand == 'time':
                        chat_history.append('Bot ' + str(time.strftime('%X %x %Z')))
                    elif rand == 'forecast':
                        df = pd.read_csv('Prediction/data/topics.csv', index_col='Date', parse_dates=True)
                        df = df.dropna()

                        topic_1 = ARIMAResults.load('Prediction/models/covid.pkl')
                        topic_2 = ARIMAResults.load('Prediction/models/gas.pkl')
                        topic_3 = ARIMAResults.load('Prediction/models/politics.pkl')
                        topic_4 = ARIMAResults.load('Prediction/models/yohani.pkl')
                        topic_5 = ARIMAResults.load('Prediction/models/travelling.pkl')
                        topic_6 = ARIMAResults.load('Prediction/models/youtube.pkl')
                        index_future_dates = pd.date_range(start=str(start_date), end=str(end_date))

                        pred_1 = topic_1.predict(start=len(df), end=len(df) + 6, typ='levels').rename(
                            'ARIMA Predictions')
                        pred_2 = topic_2.predict(start=len(df), end=len(df) + 6, typ='levels').rename(
                            'ARIMA Predictions')
                        pred_3 = topic_3.predict(start=len(df), end=len(df) + 6, typ='levels').rename(
                            'ARIMA Predictions')
                        pred_4 = topic_4.predict(start=len(df), end=len(df) + 6, typ='levels').rename(
                            'ARIMA Predictions')
                        pred_5 = topic_5.predict(start=len(df), end=len(df) + 6, typ='levels').rename(
                            'ARIMA Predictions')
                        pred_6 = topic_6.predict(start=len(df), end=len(df) + 6, typ='levels').rename(
                            'ARIMA Predictions')

                        pred_1.index = index_future_dates
                        pred_2.index = index_future_dates
                        pred_3.index = index_future_dates
                        pred_4.index = index_future_dates
                        pred_5.index = index_future_dates
                        pred_6.index = index_future_dates

                        return_value = []
                        value1 = 0
                        value2 = 0
                        value3 = 0
                        value4 = 0
                        value5 = 0
                        value6 = 0
                        for i in range(0, 7):
                            value1 += pred_1[i]
                            value2 += pred_2[i]
                            value3 += pred_3[i]
                            value4 += pred_3[i]
                            value5 += pred_3[i]
                            value6 += pred_3[i]

                        chat_history.append(
                            'Bot : ' + 'Covid : ' + str(round(value1, 2)) + ', Gas : ' + str(round(value2, 2)) + ', Politics : ' + str(round(value3, 2)))
                    else:
                        chat_history.append('Bot : ' + str(rand))
                    return render_template('chat_bot.html', reply=chat_history)
        else:
            print(f"{bot_name}: I do not understand...")
    return render_template('chat_bot.html')


@app.route('/profile')
def profile():
    user = db.get_user_by_email(email=current_user)

    if len(user) < 1:
        return render_template('login.html')

    return render_template('profile.html', email=user[0].get_email(), name=user[0].get_name(), user_id=user[0].get_id())


@app.route('/payment')
def payment():
    return render_template('payment.html')


@app.route('/face_detection')
def face_detection():
    return Response(detect_face(), mimetype='multipart/x-mixed-replace; boundary=frame')


# Actions
@app.route('/login_action', methods=['POST'])
def login_action():
    email = request.form['email']
    password = request.form['password']

    if db.login(email=email, password=password):
        global current_user
        current_user = email
        return render_template('home.html')
    else:
        return render_template('login.html', status='Username Or Password Invalid')


@app.route('/register_action', methods=['POST'])
def register_action():
    name = request.form['name']
    email = request.form['email']
    password = request.form['password']
    cpassword = request.form['cpassword']

    if password == cpassword:
        db.save_user(name=name, email=email, password=password)
        return render_template('login.html', status='Register Success')
    else:
        return render_template('register.html', status='Password did not match')


if __name__ == "__main__":
    app.run(debug=True)
